/*
 * factory.h
 *
 * Copyright (c) 2003 Frerich Raabe <raabe@kde.org>
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. For licensing and distribution details, check the
 * accompanying file 'COPYING'.
 */
#ifndef LIBRSS_FACTORY_H
#define LIBRSS_FACTORY_H

class QDomNode;

namespace RSS
{
	class Article;
	class Image;
	class TextInput;

	/**
	 * This factory is responsible for producing the objects which represent
	 * the components of which a RSS file consists, e.g. articles, images and
	 * text input objects. The only user of this class is the Document class;
	 * instead of hardcoding the type of e.g. Article to instantiate, this
	 * factory is used so that it's possible to plug in custom components.
	 *
	 * For example, to plug a new Article object which has a 'read' flag, do
	 * something like this:
	 *
	 * First, create the new Article:
	 *
	 * \code
	 * class MyArticle : public Article {
	 *   public:
	 *     MyArticle(const QDomNode &node) : Article(node), m_read(false) { }
	 *
	 *     bool read() const { return m_read; }
	 *     void setRead(bool read) { m_read = read; }
	 *
	 *   private:
	 *     bool m_read;
	 * };
	 * \endcode
	 *
	 * Now, create a subclass of the default RSS::Factory and reimplement the
	 * createArticle method so that it returns a MyArticle instance instead
	 * of the default RSS::Article:
	 *
	 * \code
	 * class MyFactory : public Factory {
	 *   public:
	 *     virtual Article *createArticle(const QDomNode &node) {
	 *        return new MyArticle(node);
	 *     }
	 * };
	 * \endcode
	 *
	 * Finally, tell the RSS::Document classes to use your new factory instead
	 * of the default one, by using this simple call:
	 *
	 * \code
	 * RSS::Document::useFactory(new MyFactory);
	 * \endcode
	 *
	 * @see RSS::Document::useFactory
	 */
	class Factory
	{
		public:
			/**
			 * Destructor. Does nothing by default.
			 */
			virtual ~Factory();

			/**
			 * @return an article created from the given DOM node; reimplement
			 * this method to plug your own subclasses of RSS::Article into the
			 * library.
			 * The ownership of the returned pointer is transferred to the
			 * caller.
			 * @param n The DOM node to construct the article from.
			 */
			virtual Article *createArticle(const QDomNode &n) const;

			/**
			 * @return an image created from the given DOM node; reimplement
			 * this method to plug your own subclasses of RSS::Image into the
			 * library.
			 * The ownership of the returned pointer is transferred to the
			 * caller.
			 * @param n The DOM node to construct the image from.
			 */
			virtual Image *createImage(const QDomNode &n) const;

			/**
			 * @return a text input created from the given DOM node; reimplement
			 * this method to plug your own subclasses of RSS::TextInput into
			 * the library.
			 * The ownership of the returned pointer is transferred to the
			 * caller.
			 * @param n The DOM node to construct the text input from.
			 */
			virtual TextInput *createTextInput(const QDomNode &n) const;
	};
}

#endif // LIBRSS_FACTORY_H
// vim: noet:ts=4
